
//// UN-TWEAKABLES - AUTOMATICALLY-TRACKED TRANSFORMS ////////////////

float4x4 WorldITXf : WorldInverseTranspose < string UIWidget="None"; >;
float4x4 WvpXf : WorldViewProjection < string UIWidget="None"; >;
float4x4 WorldXf : World < string UIWidget="None"; >;
float4x4 ViewIXf : ViewInverse < string UIWidget="None"; >;

//// TWEAKABLE PARAMETERS ////////////////////

/// Point Lamp 0 ////////////
float3 Lamp0Pos : Position <
    string Object = "PointLight0";
    string UIName =  "Lamp 0 Position";
    string Space = "World";
> = {-0.5f,2.0f,1.25f};

float3 Lamp0Color : Specular <
    string UIName =  "Lamp 0";
    string Object = "Pointlight0";
    string UIWidget = "Color";
> = {1.0f,1.0f,1.0f};

// Ambient Light
float3 AmbiColor : Ambient <
    string UIName =  "Ambient Light";
    string UIWidget = "Color";
> = {0.07f,0.07f,0.07f};

//////// COLOR & TEXTURE /////////////////////

texture ColorTextureDay : DIFFUSE <
    string ResourceName = "EarthDecal_Day.dds";
    string UIName =  "Daylight Texture";
    string ResourceType = "2D";
>;
sampler2D ColorSamplerDay = sampler_state {
    Texture = <ColorTextureDay>;
    FILTER = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};  

texture ColorTextureNight : DIFFUSE <
    string ResourceName = "EarthDecal_MoonLit.dds";
    string UIName =  "Night Texture";
    string ResourceType = "2D";
>;
sampler2D ColorSamplerNight = sampler_state {
    Texture = <ColorTextureNight>;
    FILTER = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};  

texture NormalTexture : NORMAL <
    string ResourceName = "Earth_TSNormal.dds";
    string UIName =  "Tangent Space Normal Texture";
    string ResourceType = "2D";
>;
sampler2D NormalMapSampler = sampler_state {
    Texture = <NormalTexture>;
    FILTER = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};  

texture GlossTexture : SPECULAR <
    string ResourceName = "Earth_Spec.dds";
    string UIName =  "Tangent Space Normal Texture";
    string ResourceType = "2D";
>;
sampler2D GlossMapSampler = sampler_state {
    Texture = <GlossTexture>;
    FILTER = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};  

// shared shadow mapping supported in Cg version

//////// CONNECTOR DATA STRUCTURES ///////////

/* data from application vertex buffer */
struct appdata {
    float3 Position	: POSITION;
    float4 UV		: TEXCOORD0;
    float4 Normal	: NORMAL;
    float4 Tangent	: TANGENT0;
};

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
    float4 HPosition	: POSITION;
    float2 UV		    : TEXCOORD0;
    // The following values are passed in "World" coordinates since
    //   it tends to be the most flexible and easy for handling
    //   reflections, sky lighting, and other "global" effects.
    float3 LightDir      : TEXCOORD1;
    float3 WorldNormal	 : TEXCOORD2;
    float3 WorldTangent	 : TEXCOORD3;
    float3 WorldBinormal : TEXCOORD4;
    float3 WorldView	 : TEXCOORD5;
};
 
///////// VERTEX SHADING /////////////////////

/*********** Generic Vertex Shader ******/

vertexOutput std_VS(appdata IN) {
    vertexOutput OUT = (vertexOutput)0;
	
    OUT.WorldNormal   = mul(IN.Normal,WorldITXf).xyz;
    OUT.WorldTangent  = mul(IN.Tangent,WorldITXf).xyz;
  	// Binormale in World Space
	OUT.WorldBinormal    = cross(OUT.WorldNormal,OUT.WorldTangent).xyz;

    float4 Position = float4(IN.Position.xyz,1);
    float3 PosWorld = mul(Position,WorldXf).xyz;
	
    OUT.LightDir = float4(Lamp0Pos,0);
    OUT.UV = float2(IN.UV.x,(1.0-IN.UV.y));
	
    OUT.WorldView = normalize(ViewIXf[3].xyz - PosWorld);
    OUT.HPosition = mul(Position,WvpXf);
    return OUT;
}

///////// PIXEL SHADING //////////////////////

float4 std_PS(vertexOutput IN) : COLOR {
	
   	// compute matrix for tangent space transformation 
	float3x3 matWorldToTangentSpace;
	matWorldToTangentSpace[0] = normalize(IN.WorldTangent);
	matWorldToTangentSpace[1] = normalize(IN.WorldBinormal);
	matWorldToTangentSpace[2] = normalize(IN.WorldNormal);
	
	float3 dayColor = tex2D(ColorSamplerDay, IN.UV).xyz;
	float3 nightColor = tex2D(ColorSamplerNight, IN.UV).xyz;
    float3 normalMap = tex2D(NormalMapSampler, IN.UV).xyz * 2.0 - 1.0;
	float3 glossMap = tex2D(GlossMapSampler, IN.UV).xyz;
	 
	// compute parameters N,L,V,H 
    float3 Nn = normalize(mul(normalMap,matWorldToTangentSpace));
    float3 Ln = normalize(IN.LightDir);
	float3 Vn = normalize(IN.WorldView);
	float3 Hn = normalize(Vn+Ln);
	
    float ldn = max(dot(Ln,Nn),0.0);
    float hdn = max(dot(Hn,Nn),0.0);
	
    float3 result = lerp(nightColor,dayColor,ldn);
    result += glossMap*pow(hdn,200) * Lamp0Color;
	
    return float4(result,1);
}

float4 simpleBump_PS(vertexOutput IN) : COLOR {
	
   	// compute matrix for tangent space transformation 
	float3x3 matWorldToTangentSpace;
	matWorldToTangentSpace[0] = normalize(IN.WorldTangent);
	matWorldToTangentSpace[1] = normalize(IN.WorldBinormal);
	matWorldToTangentSpace[2] = normalize(IN.WorldNormal);
	
	float3 Nn = normalize(IN.WorldNormal);
    float3 Ln = normalize(IN.LightDir);
    float3 Lt = normalize(mul(float4(Ln,1),matWorldToTangentSpace));
	
	float bumpMapA = tex2D(GlossMapSampler, IN.UV).x;
	float bumpMapB = tex2D(GlossMapSampler, IN.UV+0.002*Lt.xy).x;
	 
    float ldn = max(dot(Ln,Nn),0.0);
	
    float3 result = ldn;
    result += (bumpMapA-bumpMapB) * Lamp0Color;
	
    return float4(result,1);
}

///// TECHNIQUES /////////////////////////////

RasterizerState DisableCulling
{
    CullMode = NONE;
};

DepthStencilState DepthEnabling
{
	DepthEnable = TRUE;
};

BlendState DisableBlend
{
	BlendEnable[0] = FALSE;
};

technique10 TangentSpaceNormalMap <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        SetVertexShader( CompileShader( vs_4_0, std_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, std_PS() ) );
                
        SetRasterizerState(DisableCulling);       
		SetDepthStencilState(DepthEnabling, 0);
		SetBlendState(DisableBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}

technique10 SimpleBump10 <
	string Script = "Pass=p0;";
> {
    pass p0 <
	string Script = "Draw=geometry;";
    > {
        SetVertexShader( CompileShader( vs_4_0, std_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, simpleBump_PS() ) );
                
        SetRasterizerState(DisableCulling);       
		SetDepthStencilState(DepthEnabling, 0);
		SetBlendState(DisableBlend, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF);
    }
}


/////////////////////////////////////// eof //
